# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V21::Enums::MimeTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED  => "UNSPECIFIED",
  UNKNOWN      => "UNKNOWN",
  IMAGE_JPEG   => "IMAGE_JPEG",
  IMAGE_GIF    => "IMAGE_GIF",
  IMAGE_PNG    => "IMAGE_PNG",
  FLASH        => "FLASH",
  TEXT_HTML    => "TEXT_HTML",
  PDF          => "PDF",
  MSWORD       => "MSWORD",
  MSEXCEL      => "MSEXCEL",
  RTF          => "RTF",
  AUDIO_WAV    => "AUDIO_WAV",
  AUDIO_MP3    => "AUDIO_MP3",
  HTML5_AD_ZIP => "HTML5_AD_ZIP"
];

1;
