# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V20::Enums::PromotionPlaceholderFieldEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED        => "UNSPECIFIED",
  UNKNOWN            => "UNKNOWN",
  PROMOTION_TARGET   => "PROMOTION_TARGET",
  DISCOUNT_MODIFIER  => "DISCOUNT_MODIFIER",
  PERCENT_OFF        => "PERCENT_OFF",
  MONEY_AMOUNT_OFF   => "MONEY_AMOUNT_OFF",
  PROMOTION_CODE     => "PROMOTION_CODE",
  ORDERS_OVER_AMOUNT => "ORDERS_OVER_AMOUNT",
  PROMOTION_START    => "PROMOTION_START",
  PROMOTION_END      => "PROMOTION_END",
  OCCASION           => "OCCASION",
  FINAL_URLS         => "FINAL_URLS",
  FINAL_MOBILE_URLS  => "FINAL_MOBILE_URLS",
  TRACKING_URL       => "TRACKING_URL",
  LANGUAGE           => "LANGUAGE",
  FINAL_URL_SUFFIX   => "FINAL_URL_SUFFIX"
];

1;
