# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V21::Enums::ExperimentMetricEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                      => "UNSPECIFIED",
  UNKNOWN                          => "UNKNOWN",
  CLICKS                           => "CLICKS",
  IMPRESSIONS                      => "IMPRESSIONS",
  COST                             => "COST",
  CONVERSIONS_PER_INTERACTION_RATE => "CONVERSIONS_PER_INTERACTION_RATE",
  COST_PER_CONVERSION              => "COST_PER_CONVERSION",
  CONVERSIONS_VALUE_PER_COST       => "CONVERSIONS_VALUE_PER_COST",
  AVERAGE_CPC                      => "AVERAGE_CPC",
  CTR                              => "CTR",
  INCREMENTAL_CONVERSIONS          => "INCREMENTAL_CONVERSIONS",
  COMPLETED_VIDEO_VIEWS            => "COMPLETED_VIDEO_VIEWS",
  CUSTOM_ALGORITHMS                => "CUSTOM_ALGORITHMS",
  CONVERSIONS                      => "CONVERSIONS",
  CONVERSION_VALUE                 => "CONVERSION_VALUE"
];

1;
