# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V21::Enums::ServedAssetFieldTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                 => "UNSPECIFIED",
  UNKNOWN                     => "UNKNOWN",
  HEADLINE_1                  => "HEADLINE_1",
  HEADLINE_2                  => "HEADLINE_2",
  HEADLINE_3                  => "HEADLINE_3",
  DESCRIPTION_1               => "DESCRIPTION_1",
  DESCRIPTION_2               => "DESCRIPTION_2",
  HEADLINE                    => "HEADLINE",
  HEADLINE_IN_PORTRAIT        => "HEADLINE_IN_PORTRAIT",
  LONG_HEADLINE               => "LONG_HEADLINE",
  DESCRIPTION                 => "DESCRIPTION",
  DESCRIPTION_IN_PORTRAIT     => "DESCRIPTION_IN_PORTRAIT",
  BUSINESS_NAME_IN_PORTRAIT   => "BUSINESS_NAME_IN_PORTRAIT",
  BUSINESS_NAME               => "BUSINESS_NAME",
  MARKETING_IMAGE             => "MARKETING_IMAGE",
  MARKETING_IMAGE_IN_PORTRAIT => "MARKETING_IMAGE_IN_PORTRAIT",
  SQUARE_MARKETING_IMAGE      => "SQUARE_MARKETING_IMAGE",
  PORTRAIT_MARKETING_IMAGE    => "PORTRAIT_MARKETING_IMAGE",
  LOGO                        => "LOGO",
  LANDSCAPE_LOGO              => "LANDSCAPE_LOGO",
  CALL_TO_ACTION              => "CALL_TO_ACTION",
  YOU_TUBE_VIDEO              => "YOU_TUBE_VIDEO",
  SITELINK                    => "SITELINK",
  CALL                        => "CALL",
  MOBILE_APP                  => "MOBILE_APP",
  CALLOUT                     => "CALLOUT",
  STRUCTURED_SNIPPET          => "STRUCTURED_SNIPPET",
  PRICE                       => "PRICE",
  PROMOTION                   => "PROMOTION",
  AD_IMAGE                    => "AD_IMAGE",
  LEAD_FORM                   => "LEAD_FORM",
  BUSINESS_LOGO               => "BUSINESS_LOGO",
  DESCRIPTION_PREFIX          => "DESCRIPTION_PREFIX"
];

1;
