# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V19::Enums::ConversionOrAdjustmentLagBucketEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                          => "UNSPECIFIED",
  UNKNOWN                              => "UNKNOWN",
  CONVERSION_LESS_THAN_ONE_DAY         => "CONVERSION_LESS_THAN_ONE_DAY",
  CONVERSION_ONE_TO_TWO_DAYS           => "CONVERSION_ONE_TO_TWO_DAYS",
  CONVERSION_TWO_TO_THREE_DAYS         => "CONVERSION_TWO_TO_THREE_DAYS",
  CONVERSION_THREE_TO_FOUR_DAYS        => "CONVERSION_THREE_TO_FOUR_DAYS",
  CONVERSION_FOUR_TO_FIVE_DAYS         => "CONVERSION_FOUR_TO_FIVE_DAYS",
  CONVERSION_FIVE_TO_SIX_DAYS          => "CONVERSION_FIVE_TO_SIX_DAYS",
  CONVERSION_SIX_TO_SEVEN_DAYS         => "CONVERSION_SIX_TO_SEVEN_DAYS",
  CONVERSION_SEVEN_TO_EIGHT_DAYS       => "CONVERSION_SEVEN_TO_EIGHT_DAYS",
  CONVERSION_EIGHT_TO_NINE_DAYS        => "CONVERSION_EIGHT_TO_NINE_DAYS",
  CONVERSION_NINE_TO_TEN_DAYS          => "CONVERSION_NINE_TO_TEN_DAYS",
  CONVERSION_TEN_TO_ELEVEN_DAYS        => "CONVERSION_TEN_TO_ELEVEN_DAYS",
  CONVERSION_ELEVEN_TO_TWELVE_DAYS     => "CONVERSION_ELEVEN_TO_TWELVE_DAYS",
  CONVERSION_TWELVE_TO_THIRTEEN_DAYS   => "CONVERSION_TWELVE_TO_THIRTEEN_DAYS",
  CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS =>
    "CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS",
  CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS =>
    "CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS",
  CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS =>
    "CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS",
  CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS =>
    "CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS",
  CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS  => "CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS",
  CONVERSION_SIXTY_TO_NINETY_DAYS      => "CONVERSION_SIXTY_TO_NINETY_DAYS",
  ADJUSTMENT_LESS_THAN_ONE_DAY         => "ADJUSTMENT_LESS_THAN_ONE_DAY",
  ADJUSTMENT_ONE_TO_TWO_DAYS           => "ADJUSTMENT_ONE_TO_TWO_DAYS",
  ADJUSTMENT_TWO_TO_THREE_DAYS         => "ADJUSTMENT_TWO_TO_THREE_DAYS",
  ADJUSTMENT_THREE_TO_FOUR_DAYS        => "ADJUSTMENT_THREE_TO_FOUR_DAYS",
  ADJUSTMENT_FOUR_TO_FIVE_DAYS         => "ADJUSTMENT_FOUR_TO_FIVE_DAYS",
  ADJUSTMENT_FIVE_TO_SIX_DAYS          => "ADJUSTMENT_FIVE_TO_SIX_DAYS",
  ADJUSTMENT_SIX_TO_SEVEN_DAYS         => "ADJUSTMENT_SIX_TO_SEVEN_DAYS",
  ADJUSTMENT_SEVEN_TO_EIGHT_DAYS       => "ADJUSTMENT_SEVEN_TO_EIGHT_DAYS",
  ADJUSTMENT_EIGHT_TO_NINE_DAYS        => "ADJUSTMENT_EIGHT_TO_NINE_DAYS",
  ADJUSTMENT_NINE_TO_TEN_DAYS          => "ADJUSTMENT_NINE_TO_TEN_DAYS",
  ADJUSTMENT_TEN_TO_ELEVEN_DAYS        => "ADJUSTMENT_TEN_TO_ELEVEN_DAYS",
  ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS     => "ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS",
  ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS   => "ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS",
  ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS =>
    "ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS",
  ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS =>
    "ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS",
  ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS =>
    "ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS",
  ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS =>
    "ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS",
  ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS => "ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS",
  ADJUSTMENT_SIXTY_TO_NINETY_DAYS     => "ADJUSTMENT_SIXTY_TO_NINETY_DAYS",
  ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS =>
    "ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS",
  CONVERSION_UNKNOWN => "CONVERSION_UNKNOWN",
  ADJUSTMENT_UNKNOWN => "ADJUSTMENT_UNKNOWN"
];

1;
