# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V21::Enums::ShoppingAddProductsToCampaignRecommendationEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                                       => "UNSPECIFIED",
  UNKNOWN                                           => "UNKNOWN",
  MERCHANT_CENTER_ACCOUNT_HAS_NO_SUBMITTED_PRODUCTS =>
    "MERCHANT_CENTER_ACCOUNT_HAS_NO_SUBMITTED_PRODUCTS",
  MERCHANT_CENTER_ACCOUNT_HAS_NO_SUBMITTED_PRODUCTS_IN_FEED =>
    "MERCHANT_CENTER_ACCOUNT_HAS_NO_SUBMITTED_PRODUCTS_IN_FEED",
  ADS_ACCOUNT_EXCLUDES_OFFERS_FROM_CAMPAIGN =>
    "ADS_ACCOUNT_EXCLUDES_OFFERS_FROM_CAMPAIGN",
  ALL_PRODUCTS_ARE_EXCLUDED_FROM_CAMPAIGN =>
    "ALL_PRODUCTS_ARE_EXCLUDED_FROM_CAMPAIGN"
];

1;
