# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V21::Enums::BiddingStrategyTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED               => "UNSPECIFIED",
  UNKNOWN                   => "UNKNOWN",
  COMMISSION                => "COMMISSION",
  ENHANCED_CPC              => "ENHANCED_CPC",
  FIXED_CPM                 => "FIXED_CPM",
  INVALID                   => "INVALID",
  MANUAL_CPA                => "MANUAL_CPA",
  MANUAL_CPC                => "MANUAL_CPC",
  MANUAL_CPM                => "MANUAL_CPM",
  MANUAL_CPV                => "MANUAL_CPV",
  MAXIMIZE_CONVERSIONS      => "MAXIMIZE_CONVERSIONS",
  MAXIMIZE_CONVERSION_VALUE => "MAXIMIZE_CONVERSION_VALUE",
  PAGE_ONE_PROMOTED         => "PAGE_ONE_PROMOTED",
  PERCENT_CPC               => "PERCENT_CPC",
  TARGET_CPA                => "TARGET_CPA",
  TARGET_CPM                => "TARGET_CPM",
  TARGET_CPV                => "TARGET_CPV",
  TARGET_IMPRESSION_SHARE   => "TARGET_IMPRESSION_SHARE",
  TARGET_OUTRANK_SHARE      => "TARGET_OUTRANK_SHARE",
  TARGET_ROAS               => "TARGET_ROAS",
  TARGET_SPEND              => "TARGET_SPEND"
];

1;
